'use strict'

// Define Fastify Plugin
const fp = require('fastify-plugin')
const GrpcClient = require('./lib/client')

function grpcClient(fastify, opts, done) {
  const { namespace } = opts

  if (!('grpc' in fastify)) {
    fastify.decorate('grpc', {})
  }

  if (fastify.grpc[namespace]) {
    return done(new Error(`gRPC namespace already in use: ${namespace}`))
  }

  fastify.grpc[namespace] = new GrpcClient(opts)

  fastify.addHook('onClose', (instance, done) => {
    fastify.grpc[namespace].close()
    done()
  })

  done()
}

module.exports = fp(grpcClient, { name: 'fastify-grpc', fastify: '2.x' })
