'use strict'

const test = require('ava')

const fastify = require('fastify')
const path = require('path')
const validOptions = require('./fixtures/valid-options')
const grpcClientPlugin = require('..')

test('should register as fastify decorator', async t => {
  const instance = fastify()
  instance.register(grpcClientPlugin, validOptions)
  await instance.ready()

  t.not(undefined, instance.grpc)
  await instance.close()
})

test('should register many services', async t => {
  const fibonacciOptions = {
    namespace: 'fibonacci',
    service: 'Fibonacci',
    protoFile: path.join(__dirname, 'fixtures', 'fibonacci.proto')
  }

  const instance = fastify()
  instance.register(grpcClientPlugin, validOptions)
  instance.register(grpcClientPlugin, fibonacciOptions)
  await instance.ready()

  t.not(undefined, instance.grpc)
  t.not(undefined, instance.grpc.helloworld)
  t.not(undefined, instance.grpc.fibonacci)

  await instance.close()
})

test('should throw error if same service is registered twice', async t => {
  const instance = fastify()
  instance.register(grpcClientPlugin, validOptions)
  instance.register(grpcClientPlugin, validOptions)

  try {
    await instance.ready()

    throw new Error('Register two services with same namespace should not be allowed')
  } catch (err) {
    t.is('gRPC namespace already in use: helloworld', err.message)
  } finally {
    await instance.close()
  }
})
